@@ -38,6 +38,10 @@
 
 namespace rcsc {
 
+/*!
+  \class AngleDeg
+  \brief degree wrapper class
+*/
 class AngleDeg {
     // : public boost::addable< AngleDeg >
     // , public boost::addable2< AngleDeg, double >
@@ -52,6 +56,7 @@ private:
     double M_degree;
 
 public:
+    //! epsilon value
     static const double EPSILON;
 
     //! constant variable to convert DEGREE to RADIAN.
@@ -65,6 +70,7 @@ public:
       { }
     /*!
       \brief constructor with value.
+
       NO explicit.
       This means that we can use this class as follows.
       AngleDeg angle = 3.0;
@@ -75,8 +81,11 @@ public:
           normalize();
       }
 
-    //! operator substitution.
-    inline
+    /*!
+      \brief operator substitution.
+      \param deg initialization parameter
+      \return const reference to itself
+    */
     const
     AngleDeg & operator=( const double & deg )
       {
@@ -87,9 +96,8 @@ public:
 private:
     /*!
       \brief normalize the value to [-180, 180].
-      \return const reference to this.
+      \return const reference to itself.
     */
-    inline
     const
     AngleDeg & normalize()
       {
@@ -105,21 +113,29 @@ private:
       }
 
 public:
-    //! get const reference to M_degree.
-    inline
+
+    /*!
+      \brief get value of this angle
+      \return const reference to the member variable
+     */
     const
     double & degree() const
       {
           return M_degree;
       }
-    //! get absolute value of M_degree.
-    inline
+
+    /*!
+      \brief get absolute value of this angle
+      \return absolute value of degree()
+     */
     double abs() const
       {
           return std::fabs( degree() );
       }
-    //! get RADIAN value of M_degree.
-    inline
+    /*!
+      \brief get RADIAN value.
+      \return radian value of degree()
+     */
     double radian() const
       {
           return degree() * DEG2RAD;
@@ -132,64 +148,97 @@ public:
       }
     */
 
-    //! get new AngleDeg that M_degree is multiplyed by -1.
-    inline
+    /*!
+      \brief get new AngleDeg multiplyed by -1.
+      \return new AngleDeg object
+     */
     AngleDeg operator-() const
       {
           return AngleDeg( - degree() );
       }
-    //! operator +=
-    inline
+
+    /*!
+      \brief operator += with AngleDeg
+      \param angle added value
+      \return const reference to this
+    */
     const
-    AngleDeg & operator+=( const AngleDeg & ang )
+    AngleDeg & operator+=( const AngleDeg & angle )
       {
-          M_degree += ang.degree();
+          M_degree += angle.degree();
           return normalize();
       }
-    //! operator +=
-    inline
+
+    /*!
+      \brief operator += with double
+      \param deg added value
+      \return const reference to this
+    */
     const
     AngleDeg & operator+=( const double & deg )
       {
           M_degree += deg;
           return normalize();
       }
-    //! operator -= with AngleDeg
-    inline
+
+    /*!
+      \brief operator -= with AngleDeg
+      \param angle subtract argument
+      \return const reference to this
+     */
     const
-    AngleDeg & operator-=( const AngleDeg & ang )
+    AngleDeg & operator-=( const AngleDeg & angle )
       {
-          M_degree -= ang.degree();
+          M_degree -= angle.degree();
           return normalize();
       }
-    //! operator -= with double
-    inline
+
+    /*!
+      \brief operator -= with double
+      \param deg subtract argument
+      \return const reference to this
+     */
     const
     AngleDeg & operator-=( const double & deg )
       {
           M_degree -= deg;
           return normalize();
       }
-    //! operator *=
-    inline
+
+    /*!
+      \brief operator *=
+      \param scalar multiply argument
+      \return const reference to this
+     */
     const
     AngleDeg & operator*=( const double & scalar )
       {
           M_degree *= scalar;
           return normalize();
       }
-    //! operator /=
-    inline
+
+    /*!
+      \brief operator /=
+      \param scalar division argument
+      \return const reference to this
+     */
     const
     AngleDeg & operator/=( const double & scalar )
       {
+          if ( std::fabs( scalar ) < EPSILON )
+          {
+              return * this;
+          }
           M_degree /= scalar;
           return normalize();
       }
 
     //////////////////////////////////////////////////////////////////
 
-    inline
+    /*!
+      \brief check if this angle is left of 'angle'
+      \return true or false
+     */
     bool isLeftOf( const AngleDeg & angle ) const
       {
           //return (*this - angle).degree() < 0.0;
@@ -198,7 +247,10 @@ public:
                    || diff < -180.0 );
       }
 
-    inline
+    /*!
+      \brief check if this angle is left or equal of 'angle'
+      \return true or false
+     */
     bool isLeftEqualOf( const AngleDeg & angle ) const
       {
           //return (*this - angle).degree() <= 0.0;
@@ -207,7 +259,10 @@ public:
                    || diff < -180.0 );
       }
 
-    inline
+    /*!
+      \brief check if this angle is right of 'angle'
+      \return true or false
+     */
     bool isRightOf( const AngleDeg & angle ) const
       {
           //return (*this - angle).degree() > 0.0;
@@ -216,7 +271,10 @@ public:
                    || diff < -180.0 );
       }
 
-    inline
+    /*!
+      \brief check if this angle is right or equal of 'angle'
+      \return true or false
+     */
     bool isRightEqualOf( const AngleDeg & angle ) const
       {
           //return (*this - angle).degree() >= 0.0;
@@ -225,49 +283,68 @@ public:
                    || diff < -180.0 );
       }
 
-    //! get cosine value
-    inline
+    /*!
+      \brief calculate cosine
+      \return cosine value
+     */
     double cos() const
       {
           return std::cos( degree() * DEG2RAD );
       }
-    //! get sine value
-    inline
+
+    /*!
+      \brief calculate sine
+      \return sine value
+     */
     double sin() const
       {
           return std::sin( degree() * DEG2RAD );
       }
-    //! get tangent value
-    inline
+
+    /*!
+      \brief calculate tarngetn
+      \return tangent value
+     */
     double tan() const
       {
           return std::tan( degree() * DEG2RAD );
       }
 
-    //! check if this is within from left to right <-- turn clockwise
-    bool isWithin( const AngleDeg & left, const AngleDeg & right ) const;
+    /*!
+      \brief check if this angle is within [left, right] (turn clockwise).
+      \param left left angle
+      \param right right angle
+      \return true if this is within [left, right]. else false
+     */
+    bool isWithin( const AngleDeg & left,
+                   const AngleDeg & right ) const;
 
     /*!
-      \brief get bisect angle, but... *** Use carefully! ***
-      \param angle result is the bisect between '*this' to 'angle'
-      \return bisect angle
+      \brief calculate min/max sine value with angle error.
+      \param angle_err error value of angle
+      \param minsin pointer to the solution variable to store the min value
+      \param maxsin pointer to the solution variable to store the max value
     */
-    inline
-    AngleDeg bisect( const AngleDeg & angle ) const
-      {
-          AngleDeg rel( angle );
-          rel -= *this;
-          return AngleDeg( *this ) += rel.degree() * 0.5;
-          //return AngleDeg(this->degree() + ((ang - *this).degree() * 0.5));
-      }
+    void sinMinMax( const double & angle_err,
+                    double * minsin,
+                    double * maxsin ) const;
+
+    /*!
+      \brief calculate min/max cosine value of angle with angle error.
+      \param angle_err error value of angle
+      \param mincos pointer to the solution variable to store the min value
+      \param maxcos pointer to the solution variable to store the max value
+    */
+    void cosMinMax( const double & angle_err,
+                    double * mincos,
+                    double * maxcos ) const;
 
     ////////////////////////////////////////////////////////
     // static utility method concerned with angle operation
 
     /*!
-      static utility.
-      angle normalization.
-      \param dir angle value to normalize
+      \brief static utility. normalize angle
+      \param dir angle value to be normalized
       \return normalized angle value
      */
     inline
@@ -286,9 +363,8 @@ public:
       }
 
     /*!
-      \brief static utility.
-      convert radian to degree
-      \param radx radian value
+      \brief static utility. convert radian to degree
+      \param rad radian value
       \return degree value
     */
     inline
@@ -297,10 +373,10 @@ public:
       {
           return rad * RAD2DEG;
       }
+
     /*!
-      \brief static utility.
-      convert degree to radian
-      \param degx degree value
+      \brief static utility. convert degree to radian
+      \param deg degree value
       \return radian value
     */
     inline
@@ -311,9 +387,8 @@ public:
       }
 
     /*!
-      \brief static utility.
-      calculate cosine value for degree angle
-      \param degx degree value
+      \brief static utility. calculate cosine value for degree angle
+      \param deg degree value
       \return cosine value
     */
     inline
@@ -324,9 +399,8 @@ public:
       }
 
     /*!
-      \brief static utility.
-      calculate sine value for degree angle
-      \param degx degree value
+      \brief static utility. calculate sine value for degree angle
+      \param deg degree value
       \return sine value
     */
     inline
@@ -337,9 +411,8 @@ public:
       }
 
     /*!
-      \brief static utility.
-      calculate tangent value for degree angle
-      \param degx degree value
+      \brief static utility. calculate tangent value for degree angle
+      \param deg degree value
       \return tangent value
     */
     inline
@@ -350,10 +423,9 @@ public:
       }
 
     /*!
-      \breif static utility.
-      calculate arc cosine value
-      \param x cosine value
-      \return arc cosine value, but this is degree.
+      \brief static utility. calculate arc cosine value
+      \param cosine cosine value
+      \return arc cosine value, that is degree type.
     */
     inline
     static
@@ -367,10 +439,9 @@ public:
       }
 
     /*!
-      \brief static utility.
-      calculate arc sine value
-      \param x sine value
-      \return arc sine value, but this is degree.
+      \brief static utility. calculate arc sine value
+      \param sine sine value
+      \return arc sine value, that is degree type.
     */
     inline
     static
@@ -384,10 +455,9 @@ public:
       }
 
     /*!
-      \brief static utility.
-      calculate arc tangent value
-      \param x tangent value
-      \return arc tangent value, but this is degree.
+      \brief static utility. calculate arc tangent value
+      \param tangent tangent value
+      \return arc tangent value, that is degree.
     */
     inline
     static
@@ -397,42 +467,60 @@ public:
       }
 
     /*!
-      \brief static utility.
-      calculate arc tangent value from XY
+      \brief static utility. calculate arc tangent value from XY
       \param y coordinate Y
       \param x coordinate X
-      \return arc tangent value, but this is degree.
+      \return arc tangent value, that is degree type.
     */
     inline
     static
-    double atan2_deg( const double & y, const double & x )
+    double atan2_deg( const double & y,
+                      const double & x )
       {
           return ( ( x == 0.0 && y == 0.0 )
                    ? 0.0
                    : rad2deg( std::atan2( y, x ) ) );
       }
 
-    //! get bisect angle.  [left, right] can take a obtuse angle.
+    /*!
+      \brief static utility that returns bisect angle of [left, right].
+      \param left left start angle
+      \param right right end angle
+      \return bisect angle
+
+      this methid can take obtuse angle.
+    */
     static
-    AngleDeg bisect( const AngleDeg & left, const AngleDeg & right );
+    AngleDeg bisect( const AngleDeg & left,
+                     const AngleDeg & right );
 
     ////////////////////////////////////////////////////////
-    //! output value
-    inline
-    std::ostream & print( std::ostream & o ) const
+
+    /*!
+      \brief output value to ostream
+      \param os reference to the output stream
+      \return reference to the output stream
+     */
+    std::ostream & print( std::ostream & os ) const
       {
-          return o << degree();
+          return os << degree();
       }
-    //! output rounded value
-    inline
-    std::ostream & printRound( std::ostream & o, const double & step = 0.1 ) const
+
+    /*!
+      \brief output rounded value to ostream
+      \param os reference to the output stream
+      \param step round step
+      \return reference to the output stream
+     */
+    std::ostream & printRound( std::ostream & os,
+                               const double & step = 0.1 ) const
       {
-          return o << rint( degree() / step ) * step;
+          return os << rint( degree() / step ) * step;
       }
 
     ////////////////////////////////////////////////////////
     /*!
-      predicate function object.
+      \brief predicate function object.
       this compares two angles by degree value
     */
     class DegreeCmp
@@ -451,7 +539,8 @@ public:
 
 ////////////////////////////////////////////////////////
 /*!
-  predicate function object.
+  \brief predicate function object.
+
   this checks if input angle is within specific range.
 */
 class AngleIsWithin
@@ -460,8 +549,10 @@ class AngleIsWithin
     const AngleDeg M_right; //!< right threshold
 public:
     //! constructor
-    AngleIsWithin( const AngleDeg & left, const AngleDeg & right )
-        : M_left( left ), M_right( right )
+    AngleIsWithin( const AngleDeg & left,
+                   const AngleDeg & right )
+        : M_left( left )
+        , M_right( right )
       { }
 
     //! operator method
@@ -488,7 +579,8 @@ public:
 inline
 const
 rcsc::AngleDeg
-operator+( const rcsc::AngleDeg & lhs, const rcsc::AngleDeg & rhs )
+operator+( const rcsc::AngleDeg & lhs,
+           const rcsc::AngleDeg & rhs )
 {
     return rcsc::AngleDeg( lhs ) += rhs;
 }
@@ -503,7 +595,8 @@ operator+( const rcsc::AngleDeg & lhs, const rcsc::AngleDeg & rhs )
 inline
 const
 rcsc::AngleDeg
-operator+( const rcsc::AngleDeg & lhs, const double & rhs )
+operator+( const rcsc::AngleDeg & lhs,
+           const double & rhs )
 {
     return rcsc::AngleDeg( lhs ) += rhs;
 }
@@ -518,7 +611,8 @@ operator+( const rcsc::AngleDeg & lhs, const double & rhs )
 inline
 const
 rcsc::AngleDeg
-operator+( const double & lhs, const rcsc::AngleDeg & rhs )
+operator+( const double & lhs,
+           const rcsc::AngleDeg & rhs )
 {
     return rcsc::AngleDeg( rhs ) += lhs;
 }
@@ -533,7 +627,8 @@ operator+( const double & lhs, const rcsc::AngleDeg & rhs )
 inline
 const
 rcsc::AngleDeg
-operator-( const rcsc::AngleDeg & lhs, const rcsc::AngleDeg & rhs )
+operator-( const rcsc::AngleDeg & lhs,
+           const rcsc::AngleDeg & rhs )
 {
     return rcsc::AngleDeg( lhs ) -= rhs;
 }
@@ -548,7 +643,8 @@ operator-( const rcsc::AngleDeg & lhs, const rcsc::AngleDeg & rhs )
 inline
 const
 rcsc::AngleDeg
-operator-( const rcsc::AngleDeg & lhs, const double & rhs )
+operator-( const rcsc::AngleDeg & lhs,
+           const double & rhs )
 {
     return rcsc::AngleDeg( lhs ) -= rhs;
 }
@@ -563,7 +659,8 @@ operator-( const rcsc::AngleDeg & lhs, const double & rhs )
 inline
 const
 rcsc::AngleDeg
-operator-( const double & lhs, const rcsc::AngleDeg & rhs )
+operator-( const double & lhs,
+           const rcsc::AngleDeg & rhs )
 {
     return rcsc::AngleDeg( lhs ) -= rhs;
 }
@@ -580,7 +677,8 @@ operator-( const double & lhs, const rcsc::AngleDeg & rhs )
  */
 inline
 bool
-operator!=( const rcsc::AngleDeg & lhs, const rcsc::AngleDeg & rhs )
+operator!=( const rcsc::AngleDeg & lhs,
+            const rcsc::AngleDeg & rhs )
 {
     return std::fabs( lhs.degree() - rhs.degree() ) > rcsc::AngleDeg::EPSILON;
 }
@@ -594,7 +692,8 @@ operator!=( const rcsc::AngleDeg & lhs, const rcsc::AngleDeg & rhs )
  */
 inline
 bool
-operator!=( const rcsc::AngleDeg & lhs, const double & rhs )
+operator!=( const rcsc::AngleDeg & lhs,
+            const double & rhs )
 {
     return std::fabs( lhs.degree() - rhs ) > rcsc::AngleDeg::EPSILON;
 }
@@ -608,7 +707,8 @@ operator!=( const rcsc::AngleDeg & lhs, const double & rhs )
  */
 inline
 bool
-operator!=( const double & lhs, const rcsc::AngleDeg & rhs )
+operator!=( const double & lhs,
+            const rcsc::AngleDeg & rhs )
 {
     return std::fabs( lhs - rhs.degree() ) > rcsc::AngleDeg::EPSILON;
 }
@@ -622,7 +722,8 @@ operator!=( const double & lhs, const rcsc::AngleDeg & rhs )
  */
 inline
 bool
-operator==( const rcsc::AngleDeg & lhs, const rcsc::AngleDeg & rhs )
+operator==( const rcsc::AngleDeg & lhs,
+            const rcsc::AngleDeg & rhs )
 {
     return std::fabs( lhs.degree() - rhs.degree() ) < rcsc::AngleDeg::EPSILON;
 }
@@ -636,7 +737,8 @@ operator==( const rcsc::AngleDeg & lhs, const rcsc::AngleDeg & rhs )
  */
 inline
 bool
-operator==( const rcsc::AngleDeg& lhs, const double& rhs )
+operator==( const rcsc::AngleDeg& lhs,
+            const double& rhs )
 {
     return std::fabs( lhs.degree() - rhs ) < rcsc::AngleDeg::EPSILON;
 }
@@ -650,26 +752,26 @@ operator==( const rcsc::AngleDeg& lhs, const double& rhs )
  */
 inline
 bool
-operator==( const double & lhs, const rcsc::AngleDeg & rhs )
+operator==( const double & lhs,
+            const rcsc::AngleDeg & rhs )
 {
     return std::fabs( lhs - rhs.degree() ) < rcsc::AngleDeg::EPSILON;
 }
 
-/////////////////////////////////////////////////////
-// stream operator
 
 /*-------------------------------------------------------------------*/
 /*!
-  output to stream
-  \param o reference to the output stream
+  \brief output to ostream
+  \param os reference to the output stream
   \param a angle object
   \return reference to the output stream
  */
 inline
 std::ostream &
-operator<<( std::ostream & o, const rcsc::AngleDeg & a )
+operator<<( std::ostream & os,
+            const rcsc::AngleDeg & a )
 {
-    return a.print( o );
+    return a.print( os );
 }
 
 #endif
