@@ -2,7 +2,7 @@
 
 /*!
   \file gz.h
-  \brief Library Main Header File.
+  \brief gzip library Header File.
 */
 
 /*
@@ -29,8 +29,8 @@
 
 /////////////////////////////////////////////////////////////////////
 
-#ifndef RCTOOLS_RCSC_GZ_H
-#define RCTOOLS_RCSC_GZ_H
+#ifndef RCSC_GZ_H
+#define RCSC_GZ_H
 
 #include <rcsc/gz/gzfstream.h>
 #include <rcsc/gz/gzfilterstream.h>
