@@ -8,7 +8,7 @@
 /*
  *Copyright:
 
- Copyright (C) Hidehisa Akiyama
+ Copyright (C) Hidehisa AKIYAMA
 
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -29,27 +29,40 @@
 
 /////////////////////////////////////////////////////////////////////
 
-#ifndef RCTOOLS_RCSC_RCG_PARSER_V1_H
-#define RCTOOLS_RCSC_RCG_PARSER_V1_H
+#ifndef RCSC_RCG_PARSER_V1_H
+#define RCSC_RCG_PARSER_V1_H
 
 #include <rcsc/rcg/parser.h>
 
 namespace rcsc {
 namespace rcg {
 
-//! rcg v1 parser class
+/*!
+  \class ParserV1
+  \brief rcg v1 parser class
+ */
 class ParserV1
     : public Parser {
 public:
 
-    //! return version number
+    /*!
+      \brief get supported rcg version
+      \return version number
+     */
     int version() const
       {
           return REC_OLD_VERSION;
       }
 
-    //! operation method
-    bool parse( std::istream & istrm, Handler & handler ) const;
+    /*!
+      \brief parse input stream
+      \param is reference to the imput stream (usually ifstream/gzifstream).
+      \param handler reference to the rcg data handler.
+      \retval true, if successfuly parsed.
+      \retval false, if incorrect format is detected.
+    */
+    bool parse( std::istream & is,
+                Handler & handler ) const;
 };
 
 } // end of namespace
