@@ -1,14 +1,14 @@
 // -*-c++-*-
 
 /*!
-  \file types.h
+  \file rcsc/rcg/types.h
   \brief rcg data format types.
 */
 
 /*
  *Copyright:
 
- Copyright (C) 2004 Hidehisa Akiyama
+ Copyright (C) Hidehisa AKIYAMA
 
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -29,82 +29,25 @@
 
 /////////////////////////////////////////////////////////////////////
 
-#ifndef RCTOOLS_RCSC_RCG_TYPES_H
-#define RCTOOLS_RCSC_RCG_TYPES_H
+#ifndef RCSC_RCG_TYPES_H
+#define RCSC_RCG_TYPES_H
+
+#include <rcsc/types.h>
 
 namespace rcsc {
 namespace rcg {
 
-//! side type id
-enum Side {
-    LEFT = 1,
-    NEUTRAL = 0,
-    RIGHT = -1,
-};
+/*!
+  \brief max length of color name string.
 
-//! playmode types
-enum PlayMode {
-    PM_Null,
-    PM_BeforeKickOff,
-    PM_TimeOver,
-    PM_PlayOn,
-    PM_KickOff_Left,
-    PM_KickOff_Right,
-    PM_KickIn_Left,
-    PM_KickIn_Right,
-    PM_FreeKick_Left,
-    PM_FreeKick_Right,
-    PM_CornerKick_Left,
-    PM_CornerKick_Right,
-    PM_GoalKick_Left,
-    PM_GoalKick_Right,
-    PM_AfterGoal_Left,
-    PM_AfterGoal_Right, // - sserver-2.94
-    PM_Drop_Ball, // - sserver-3.29
-    PM_OffSide_Left,
-    PM_OffSide_Right, // untill sserver-5.27
-    // [I.Noda:00/05/13] added for 3D viewer/commentator/small league
-    PM_PK_Left,
-    PM_PK_Right,
-    PM_FirstHalfOver,
-    PM_Pause,
-    PM_Human,
-    PM_Foul_Charge_Left,
-    PM_Foul_Charge_Right,
-    PM_Foul_Push_Left,
-    PM_Foul_Push_Right,
-    PM_Foul_MultipleAttacker_Left,
-    PM_Foul_MultipleAttacker_Right,
-    PM_Foul_BallOut_Left,
-    PM_Foul_BallOut_Right, // until sserver-7.11
-    PM_Back_Pass_Left, // after rcssserver-8.05-rel
-    PM_Back_Pass_Right,
-    PM_Free_Kick_Fault_Left,
-    PM_Free_Kick_Fault_Right,
-    PM_CatchFault_Left,
-    PM_CatchFault_Right,
-    PM_IndFreeKick_Left, // after rcssserver-9.2.0
-    PM_IndFreeKick_Right,
-    PM_PenaltySetup_Left,  // after rcssserver-9.3.0
-    PM_PenaltySetup_Right,
-    PM_PenaltyReady_Left,
-    PM_PenaltyReady_Right,
-    PM_PenaltyTaken_Left,
-    PM_PenaltyTaken_Right,
-    PM_PenaltyMiss_Left,
-    PM_PenaltyMiss_Right,
-    PM_PenaltyScore_Left,
-    PM_PenaltyScore_Right,
-    PM_MAX
-};
-
-//! max player number in one team
-const int MAX_PLAYER = 11;
-
-//! not used
+  Actually, this variable is not used.
+*/
 const int COLOR_NAME_MAX = 64;
 
-//! monitor data type identifier.
+/*!
+  \enum DispInfoMode
+  \brief monitor data type enumeration.
+ */
 enum DispInfoMode {
     NO_INFO     = 0, //!< dummy type
     SHOW_MODE   = 1, //!< showinfo_t
@@ -118,13 +61,19 @@ enum DispInfoMode {
     PPARAM_MODE = 9  //!< player_params_t
 };
 
-//! message data type id.
+/*!
+  \enum MsgInfoMode
+  \brief message data type id enumeration.
+ */
 enum MsgInfoMode {
     MSG_BOARD = 1,
     LOG_BOARD = 2
 };
 
-//! player status bit mask.
+/*!
+  \enum PlayerStatus
+  \brief player status bit mask.
+ */
 enum PlayerStatus {
     DISABLE         = 0x0000,
     STAND           = 0x0001,
@@ -146,7 +95,7 @@ enum PlayerStatus {
 };
 
 
-/////////////////////////////////////////
+/*-------------------------------------------------------------------*/
 // monitor protocol version 1
 // game log format versin 1 & version 2
 
@@ -154,7 +103,10 @@ const double SHOWINFO_SCALE = 16.0; //!< scaling parameter
 const int REC_OLD_VERSION = 1; //!< version number of rcg v1
 const int REC_VERSION_2 = 2; //!< recorded value of rcg v2
 
-//! object data
+/*!
+  \struct pos_t
+  \brief object data
+*/
 struct pos_t {
     short enable; //!< determines if this object is in field.
     short side;   //!< side type of this object
@@ -164,13 +116,19 @@ struct pos_t {
     short y;      //!< y-coordinate
 };
 
-//! team data
+/*!
+  \struct team_t
+  \brief team data
+*/
 struct team_t {
     char name[16]; //!< team name string
     short score;   //!< score
 };
 
-//! view data
+/*!
+  \struct showinfo_t
+  \brief view data of rcssmonitor v1 protocol
+*/
 struct showinfo_t {
     char pmode;     //!< playmode id
     team_t team[2]; //!< team date
@@ -178,57 +136,75 @@ struct showinfo_t {
     short time; //!< game time
 };
 
-//! message data
+/*!
+  \struct msginfo_t
+  \brief message data
+*/
 struct msginfo_t {
     short board;  //!< message type id
     char message[2048]; //!< message body
 };
 
-//! point data in drawinfo_t
+/*!
+  \struct pointinfo_t
+  \brief point data in drawinfo_t
+*/
 struct pointinfo_t {
-    short x;
-    short y;
-    char color[COLOR_NAME_MAX];
+    short x; //!< x pos
+    short y; //!< y pos
+    char color[COLOR_NAME_MAX]; //!< color name
 };
 
-//! circle data in drawinfo_t
+/*!
+  \struct circleinfo_t
+  \brief circle data in drawinfo_t
+*/
 struct circleinfo_t {
-    short x;
-    short y;
-    short r;
-    char color[COLOR_NAME_MAX];
+    short x; //!< x pos
+    short y; //!< y pos
+    short r; //!< radius
+    char color[COLOR_NAME_MAX]; //!< color name
 };
 
-//! line data in drawinfo_t
+/*!
+  \struct lineinfo_t
+  \brief line data in drawinfo_t
+*/
 struct lineinfo_t {
-    short x1;
-    short y1;
-    short x2;
-    short y2;
-    char color[COLOR_NAME_MAX];
+    short x1; //!< x of point1
+    short y1; //!< y of point1
+    short x2; //!< x of point2
+    short y2; //!< y of point2
+    char color[COLOR_NAME_MAX]; //!< color name
 };
 
-//! drawing data
+/*!
+  \struct drawinfo_t
+  \brief drawing data
+*/
 struct drawinfo_t {
     short mode; //!< data type id
     union {
         pointinfo_t pinfo;
         circleinfo_t cinfo;
         lineinfo_t linfo;
-    } object;
+    } object; //!< union variable
 };
 
-//! data block for rcg v1/v2
+/*!
+  \struct dispinfo_t
+  \brief data block for rcssmonitor v1 protocol and rcg v1/v2
+*/
 struct dispinfo_t {
     short mode; //!< data type id
     union {
-        showinfo_t show;
-        msginfo_t msg;
-        drawinfo_t draw;
-    } body;
+        showinfo_t show; //!< view data
+        msginfo_t msg; //!< message data
+        drawinfo_t draw; //!< draw data
+    } body; //!< union variable
 };
 
-/////////////////////////////////////////
+/*-------------------------------------------------------------------*/
 // monitor protocol version 2
 // game log format version 3
 
@@ -237,7 +213,10 @@ const double SHOWINFO_SCALE2 = 65536.0;
 //! version number
 const int REC_VERSION_3 = 3;
 
-//! ball data
+/*!
+  \struct ball_t
+  \brief ball data
+*/
 struct ball_t {
     long x; //!< pos x scaled by SHOWINFO_SCALE2
     long y; //!< pos y scaled by SHOWINFO_SCALE2
@@ -245,7 +224,10 @@ struct ball_t {
     long deltay; //!< velocity y scaled by SHOWINFO_SCALE2
 };
 
-//! player data
+/*!
+  \struct player_t
+  \brief player data
+*/
 struct player_t {
     short mode; //!< status flag
     short type; //!< player type id
@@ -270,7 +252,10 @@ struct player_t {
     short change_view_count; //!< executed change_view command count
 };
 
-//! view data
+/*!
+  \struct showinfo_t2
+  \brief view data of rcssmonitor v2 protocol
+*/
 struct showinfo_t2 {
     char pmode; //!< playmode id
     team_t team[2]; //!< team info
@@ -280,9 +265,10 @@ struct showinfo_t2 {
 };
 
 /*!
+  \struct short_showinfo_t2
   \brief view data.
 
-  rcg v3 includes only this as view data.
+  rcg v3 files include only this as view data.
 */
 struct short_showinfo_t2 {
     ball_t ball; //!< ball info
@@ -290,7 +276,10 @@ struct short_showinfo_t2 {
     short time; //!< game time
 };
 
-//! player type parametors
+/*!
+  \struct player_type_t
+  \brief player type parameters
+ */
 struct player_type_t {
     short id; //!< type id
     long player_speed_max; //!< max speed
@@ -305,19 +294,22 @@ struct player_type_t {
     long effort_max; //!< max(initial) effort value
     long effort_min; //!< min effort value
 
-    long sparelong1;
-    long sparelong2;
-    long sparelong3;
-    long sparelong4;
-    long sparelong5;
-    long sparelong6;
-    long sparelong7;
-    long sparelong8;
-    long sparelong9;
-    long sparelong10;
+    long sparelong1; //!< spare variable
+    long sparelong2; //!< spare variable
+    long sparelong3; //!< spare variable
+    long sparelong4; //!< spare variable
+    long sparelong5; //!< spare variable
+    long sparelong6; //!< spare variable
+    long sparelong7; //!< spare variable
+    long sparelong8; //!< spare variable
+    long sparelong9; //!< spare variable
+    long sparelong10; //!< spare variable
 };
 
-//! server parametors
+/*!
+  \struct server_params_t
+  \brief server parametors
+ */
 struct server_params_t {
     long goal_width; //!< goal width
     long inertia_moment; //!< related to player's turn action
@@ -381,12 +373,12 @@ struct server_params_t {
     short use_wind_random; //!< wind factor is random
     short coach_say_count_max; //!< max count of coach's freeform say
     short coach_say_msg_size; //!< max length of coach's freeform say message
-    short clang_win_size;
-    short clang_define_win;
-    short clang_meta_win;
-    short clang_advice_win;
-    short clang_info_win;
-    short clang_mess_delay; //!< CLang advise is delayed this cycle
+    short clang_win_size; //!< window of coach language window
+    short clang_define_win; //!< window of coach language define message
+    short clang_meta_win; //!< window of coach lauguage meta message
+    short clang_advice_win; //!< window of coach language advice message
+    short clang_info_win; //!< window of coach language info message
+    short clang_mess_delay; //!< coach language advise message is delayed this cycle
     short clang_mess_per_cycle; //!< online coach can send CLang advise per this cycle
     short half_time; //!<  half time cycle
     short simulator_step; //!< simulator step interval msec
@@ -422,12 +414,12 @@ struct server_params_t {
 
     long ka_length; //!< keep away region length
     long ka_width;  //!< keep away region width
-    long sparelong5;
-    long sparelong6;
-    long sparelong7;
-    long sparelong8;
-    long sparelong9;
-    long sparelong10;
+    long sparelong5; //!< spare variable
+    long sparelong6; //!< spare variable
+    long sparelong7; //!< spare variable
+    long sparelong8; //!< spare variable
+    long sparelong9; //!< spare variable
+    long sparelong10; //!< spare variable
 
     short start_goal_l; //!< initial left team score
     short start_goal_r; //!< initial right team score
@@ -454,53 +446,57 @@ struct server_params_t {
     */
 };
 
-//! heterogenious player trade-off parametors
+/*!
+  \struct player_params_t
+  \brief heterogenious player trade-off parametors
+*/
 struct player_params_t {
     short player_types; //!< the number of player types
     short substitute_max; //!< max player substitution
     short pt_max; //!< max each player type on field
-    long player_speed_max_delta_min;
-    long player_speed_max_delta_max;
-    long stamina_inc_max_delta_factor;
-    long player_decay_delta_min;
-    long player_decay_delta_max;
-    long inertia_moment_delta_factor;
-    long dash_power_rate_delta_min;
-    long dash_power_rate_delta_max;
-    long player_size_delta_factor;
-    long kickable_margin_delta_min;
-    long kickable_margin_delta_max;
-    long kick_rand_delta_factor;
-    long extra_stamina_delta_min;
-    long extra_stamina_delta_max;
-    long effort_max_delta_factor;
-    long effort_min_delta_factor;
-    long random_seed;
-    long new_dash_power_rate_delta_min;
-    long new_dash_power_rate_delta_max;
-    long new_stamina_inc_max_delta_factor;
+    long player_speed_max_delta_min; //!< minimal value wighin delta range
+    long player_speed_max_delta_max; //!< maximal value wighin delta range
+    long stamina_inc_max_delta_factor; //!< tradeoff parameter
+    long player_decay_delta_min; //!< minimal value wighin delta range
+    long player_decay_delta_max; //!< maximal value wighin delta range
+    long inertia_moment_delta_factor; //!< tradeoff parameter
+    long dash_power_rate_delta_min; //!< minimal value wighin delta range
+    long dash_power_rate_delta_max; //!< maximal value wighin delta range
+    long player_size_delta_factor; //!< tradeoff parameter
+    long kickable_margin_delta_min; //!< minimal value wighin delta range
+    long kickable_margin_delta_max; //!< maximal value wighin delta range
+    long kick_rand_delta_factor; //!< tradeoff parameter
+    long extra_stamina_delta_min; //!< minimal value wighin delta range
+    long extra_stamina_delta_max; //!< maximal value wighin delta range
+    long effort_max_delta_factor; //!< tradeoff parameter
+    long effort_min_delta_factor; //!< tradeoff parameter
+    long random_seed; //!< random seed to generate parameters
+    long new_dash_power_rate_delta_min; //!< minimal value wighin delta range
+    long new_dash_power_rate_delta_max; //!< maximal value wighin delta range
+    long new_stamina_inc_max_delta_factor; //!< tradeoff parameter
 
-    long sparelong5;
-    long sparelong6;
-    long sparelong7;
-    long sparelong8;
-    long sparelong9;
-    long sparelong10;
+    long sparelong5; //!< spare variable
+    long sparelong6; //!< spare variable
+    long sparelong7; //!< spare variable
+    long sparelong8; //!< spare variable
+    long sparelong9; //!< spare variable
+    long sparelong10; //!< spare variable
 
-    short spareshort1;
-    short spareshort2;
-    short spareshort3;
-    short spareshort4;
-    short spareshort5;
-    short spareshort6;
-    short spareshort7;
-    short spareshort8;
-    short spareshort9;
-    short spareshort10;
+    short spareshort1; //!< spare variable
+    short spareshort2; //!< spare variable
+    short spareshort3; //!< spare variable
+    short spareshort4; //!< spare variable
+    short spareshort5; //!< spare variable
+    short spareshort6; //!< spare variable
+    short spareshort7; //!< spare variable
+    short spareshort8; //!< spare variable
+    short spareshort9; //!< spare variable
+    short spareshort10; //!< spare variable
 };
 
 /*!
-  \brief data block for monitor v2 protocol.
+  \struct dispinfo_t2
+  \brief data block for rcssmonitor v2 protocol.
 
   This data format is used only for monitor protocol and not used in rcg.
 */
@@ -512,7 +508,7 @@ struct dispinfo_t2 {
         player_type_t ptinfo; //!< player type data
         server_params_t sparams; //!< server param data
         player_params_t pparams; //!< payer trade-off param data
-    } body;
+    } body; //!< union variable
 };
 
 //! default rcg version
